from PIL import Image
import numpy as np

# --- CONFIG ---
INPUT = "/Users/fjansen/Downloads/Table&Tulip/T&T_Logos_Final/T&T_Monogram.png"
OUTPUT_PNG = "/Users/fjansen/Downloads/Table&Tulip/T&T_Logos_Final/favicon-tight-512.png"
OUTPUT_ICO = "/Users/fjansen/Downloads/Table&Tulip/T&T_Logos_Final/favicon-tight.ico"
TARGET_SIZE = 480        # favicon base size
PADDING = 0.08           # % of edge padding after cropping


# --- LOAD IMAGE ---
img = Image.open(INPUT).convert("RGBA")
arr = np.asarray(img)

# --- DETECT NON-WHITE / NON-EMPTY PIXELS ---
# Treat nearly-white or transparent as background
mask = ~(
        ((arr[...,0] > 245) & (arr[...,1] > 245) & (arr[...,2] > 245)) |
        (arr[...,3] < 10)
)

coords = np.argwhere(mask)

# If nothing detected, fallback
if coords.size == 0:
    cropped = img
else:
    y0, x0 = coords.min(axis=0)[:2]
    y1, x1 = coords.max(axis=0)[:2] + 1
    cropped = img.crop((x0, y0, x1, y1))


# --- ADD MINIMAL PADDING ---
cw, ch = cropped.size
pad = int(max(cw, ch) * PADDING)

square_size = max(cw, ch) + pad * 2

#canvas = Image.new("RGBA", (square_size, square_size), (0,0,0,0))
#canvas.paste(cropped, ((square_size - cw)//2, (square_size - ch)//2))
# *** White background version ***
canvas = Image.new("RGBA", (square_size, square_size), (255, 255, 255, 255))
canvas.paste(cropped, ((square_size - cw)//2, (square_size - ch)//2), cropped)

# --- RESIZE FOR FAVORICON ---
favicon = canvas.resize((TARGET_SIZE, TARGET_SIZE), Image.LANCZOS)

# Save PNG
favicon.save(OUTPUT_PNG)

# Save ICO in multiple resolutions
favicon.save(
    OUTPUT_ICO,
    format="ICO",
    sizes=[(16,16),(32,32),(48,48),(64,64),(128,128),(256,256)]
)

OUTPUT_PNG, OUTPUT_ICO



"""
A Python implementation of Google's Encoded Polyline Algorithm Format.
"""
import io
import itertools
import math
from typing import List, Tuple


def _pcitr(iterable):
    return zip(iterable, itertools.islice(iterable, 1, None))


def _py2_round(x):
    # The polyline algorithm uses Python 2's way of rounding
    return int(math.copysign(math.floor(math.fabs(x) + 0.5), x))


def _write(output, curr_value, prev_value, factor):
    curr_value = _py2_round(curr_value * factor)
    prev_value = _py2_round(prev_value * factor)
    coord = curr_value - prev_value
    coord <<= 1
    coord = coord if coord >= 0 else ~coord

    while coord >= 0x20:
        output.write(chr((0x20 | (coord & 0x1f)) + 63))
        coord >>= 5

    output.write(chr(coord + 63))


def _trans(value, index):
    byte, result, shift = None, 0, 0

    comp = None
    while byte is None or byte >= 0x20:
        byte = ord(value[index]) - 63
        index += 1
        result |= (byte & 0x1f) << shift
        shift += 5
        comp = result & 1

    return ~(result >> 1) if comp else (result >> 1), index


def decode(expression: str, precision: int = 5, geojson: bool = False) -> List[Tuple[float, float]]:
    """
    Decode a polyline string into a set of coordinates.

    :param expression: Polyline string, e.g. 'u{~vFvyys@fS]'.
    :param precision: Precision of the encoded coordinates. Google Maps uses 5, OpenStreetMap uses 6.
        The default value is 5.
    :param geojson: Set output of tuples to (lon, lat), as per https://tools.ietf.org/html/rfc7946#section-3.1.1
    :return: List of coordinate tuples in (lat, lon) order, unless geojson is set to True.
    """
    coordinates, index, lat, lng, length, factor = [], 0, 0, 0, len(expression), float(10 ** precision)

    while index < length:
        lat_change, index = _trans(expression, index)
        lng_change, index = _trans(expression, index)
        lat += lat_change
        lng += lng_change
        coordinates.append((lat / factor, lng / factor))

    if geojson is True:
        coordinates = [t[::-1] for t in coordinates]

    return coordinates


def encode(coordinates: List[Tuple[float, float]], precision: int = 5, geojson: bool = False) -> str:
    """
    Encode a set of coordinates in a polyline string.

    :param coordinates: List of coordinate tuples, e.g. [(0, 0), (1, 0)]. Unless geojson is set to True, the order
        is expected to be (lat, lon).
    :param precision: Precision of the coordinates to encode. Google Maps uses 5, OpenStreetMap uses 6.
        The default value is 5.
    :param geojson: Set to True in order to encode (lon, lat) tuples.
    :return: The encoded polyline string.
    """
    if geojson is True:
        coordinates = [t[::-1] for t in coordinates]

    output, factor = io.StringIO(), int(10 ** precision)

    _write(output, coordinates[0][0], 0, factor)
    _write(output, coordinates[0][1], 0, factor)

    for prev, curr in _pcitr(coordinates):
        _write(output, curr[0], prev[0], factor)
        _write(output, curr[1], prev[1], factor)

    return output.getvalue()
